@@ -1,4 +1,4 @@
-use crate::{crossdev, get_size_or_panic, InodeFilter, WalkOptions};
+use crate::{crossdev, get_size_or_panic, InodeFilter, Throttle, WalkOptions};
 use anyhow::Result;
 use filesize::PathExt;
 use petgraph::{graph::NodeIndex, stable_graph::StableGraph, Directed, Direction};
@@ -6,7 +6,7 @@ use std::{
     fs::Metadata,
     io,
     path::{Path, PathBuf},
-    time::{Duration, Instant},
+    time::Duration,
 };
 
 pub type TreeIndex = NodeIndex;
@@ -21,10 +21,8 @@ pub struct EntryData {
     pub metadata_io_error: bool,
 }
 
-const REFRESH_RATE: Duration = Duration::from_millis(100);
-
 /// The result of the previous filesystem traversal
-#[derive(Default, Debug)]
+#[derive(Debug)]
 pub struct Traversal {
     /// A tree representing the entire filestem traversal
     pub tree: Tree,
@@ -32,6 +30,10 @@ pub struct Traversal {
     pub root_index: TreeIndex,
     /// Amount of files or directories we have seen during the filesystem traversal
     pub entries_traversed: u64,
+    /// The time at which the traversal started.
+    pub start: std::time::Instant,
+    /// The amount of time it took to finish the traversal. Set only once done.
+    pub elapsed: Option<std::time::Duration>,
     /// Total amount of IO errors encountered when traversing the filesystem
     pub io_errors: u64,
     /// Total amount of bytes seen during the traversal
@@ -64,7 +66,11 @@ impl Traversal {
             Traversal {
                 tree,
                 root_index,
-                ..Default::default()
+                entries_traversed: 0,
+                start: std::time::Instant::now(),
+                elapsed: None,
+                io_errors: 0,
+                total_bytes: None,
             }
         };
 
@@ -74,10 +80,7 @@ impl Traversal {
         let mut previous_depth = 0;
         let mut inodes = InodeFilter::default();
 
-        let mut last_checked = Instant::now();
-
-        const INITIAL_CHECK_INTERVAL: usize = 500;
-        let mut check_instant_every = INITIAL_CHECK_INTERVAL;
+        let throttle = Throttle::new(Duration::from_millis(250), None);
         if walk_options.threads == 0 {
             // avoid using the global rayon pool, as it will keep a lot of threads alive after we are done.
             // Also means that we will spin up a bunch of threads per root path, instead of reusing them.
@@ -94,7 +97,6 @@ impl Traversal {
         }
 
         for path in input.into_iter() {
-            let mut last_seen_eid = 0;
             let device_id = match crossdev::init(path.as_ref()) {
                 Ok(id) => id,
                 Err(_) => {
@@ -102,10 +104,9 @@ impl Traversal {
                     continue;
                 }
             };
-            for (eid, entry) in walk_options
+            for entry in walk_options
                 .iter_from_path(path.as_ref(), device_id)
                 .into_iter()
-                .enumerate()
             {
                 t.entries_traversed += 1;
                 let mut data = EntryData::default();
@@ -191,22 +192,8 @@ impl Traversal {
                     }
                 }
 
-                if eid != 0
-                    && eid % check_instant_every == 0
-                    && last_checked.elapsed() >= REFRESH_RATE
-                {
-                    let now = Instant::now();
-                    let elapsed = (now - last_checked).as_millis() as f64;
-                    check_instant_every = (INITIAL_CHECK_INTERVAL as f64
-                        * ((eid - last_seen_eid) as f64 / INITIAL_CHECK_INTERVAL as f64)
-                        * (REFRESH_RATE.as_millis() as f64 / elapsed))
-                        .max(1.0) as usize;
-                    last_seen_eid = eid;
-                    last_checked = now;
-
-                    if update(&mut t)? {
-                        return Ok(None);
-                    }
+                if throttle.can_update() && update(&mut t)? {
+                    return Ok(None);
                 }
             }
         }
@@ -222,6 +209,7 @@ impl Traversal {
         set_size_or_panic(&mut t.tree, t.root_index, root_size);
         t.total_bytes = Some(root_size);
 
+        t.elapsed = Some(t.start.elapsed());
         Ok(Some(t))
     }
 
